#![feature(future_join, box_as_ptr, let_chains)]

extern crate async_std;
extern crate juno_build;

use core::ptr::copy_nonoverlapping;

juno_build::juno!("matmul");

fn main() {
    async_std::task::block_on(async {
        let a: Box<[f32]> = Box::new([1.0, 2.0, 3.0, 4.0]);
        let b: Box<[f32]> = Box::new([5.0, 6.0, 7.0, 8.0]);
        let mut a_bytes: Box<[u8]> = Box::new([0; 16]);
        let mut b_bytes: Box<[u8]> = Box::new([0; 16]);
        unsafe {
            copy_nonoverlapping(
                Box::as_ptr(&a) as *const u8,
                Box::as_mut_ptr(&mut a_bytes) as *mut u8,
                16,
            );
            copy_nonoverlapping(
                Box::as_ptr(&b) as *const u8,
                Box::as_mut_ptr(&mut b_bytes) as *mut u8,
                16,
            );
        };
        let c_bytes = matmul(2, 2, 2, a_bytes, b_bytes).await;
        let mut c: Box<[f32]> = Box::new([0.0; 4]);
        unsafe {
            copy_nonoverlapping(
                Box::as_ptr(&c_bytes) as *const u8,
                Box::as_mut_ptr(&mut c) as *mut u8,
                16,
            );
        };
        println!("[[{}, {}], [{}, {}]]", c[0], c[1], c[2], c[3]);
        assert_eq!(c[0], 19.0);
        assert_eq!(c[1], 22.0);
        assert_eq!(c[2], 43.0);
        assert_eq!(c[3], 50.0);
    });
}

#[test]
fn matmul_test() {
    main();
}

